from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.filesystem_config import FilesystemConfig
from ..models.user_filters import UserFilters
from ..models.user_oidc_custom_fields import UserOidcCustomFields
from ..models.user_permissions import UserPermissions
from ..models.user_status import UserStatus
from ..models.virtual_folder import VirtualFolder
from ..types import UNSET, Unset

T = TypeVar("T", bound="User")


@attr.s(auto_attribs=True)
class User:
    """
    Attributes:
        id (Union[Unset, int]):
        status (Union[Unset, UserStatus]): status:
              * `0` user is disabled, login is not allowed
              * `1` user is enabled
        username (Union[Unset, str]): username is unique
        email (Union[Unset, str]):
        description (Union[Unset, str]): optional description, for example the user full name
        expiration_date (Union[Unset, int]): expiration date as unix timestamp in milliseconds. An expired account
            cannot login. 0 means no expiration
        password (Union[Unset, str]): password or public key/SSH user certificate are mandatory. If the password has no
            known hashing algo prefix it will be stored, by default, using bcrypt, argon2id is supported too. You can send a
            password hashed as bcrypt ($2a$ prefix), argon2id, pbkdf2 or unix crypt and it will be stored as is. For
            security reasons this field is omitted when you search/get users
        public_keys (Union[Unset, List[str]]): Public keys in OpenSSH format. A password or at least one public key/SSH
            user certificate are mandatory.
        home_dir (Union[Unset, str]): path to the user home directory. The user cannot upload or download files outside
            this directory. SFTPGo tries to automatically create this folder if missing. Must be an absolute path
        virtual_folders (Union[Unset, List[VirtualFolder]]): mapping between virtual SFTPGo paths and filesystem paths
            outside the user home directory. Supported for local filesystem only. If one or more of the specified folders
            are not inside the dataprovider they will be automatically created. You have to create the folder on the
            filesystem yourself
        uid (Union[Unset, int]): if you run SFTPGo as root user, the created files and directories will be assigned to
            this uid. 0 means no change, the owner will be the user that runs SFTPGo. Ignored on windows
        gid (Union[Unset, int]): if you run SFTPGo as root user, the created files and directories will be assigned to
            this gid. 0 means no change, the group will be the one of the user that runs SFTPGo. Ignored on windows
        max_sessions (Union[Unset, int]): Limit the sessions that a user can open. 0 means unlimited
        quota_size (Union[Unset, int]): Quota as size in bytes. 0 menas unlimited. Please note that quota is updated if
            files are added/removed via SFTPGo otherwise a quota scan or a manual quota update is needed
        quota_files (Union[Unset, int]): Quota as number of files. 0 menas unlimited. Please note that quota is updated
            if files are added/removed via SFTPGo otherwise a quota scan or a manual quota update is needed
        permissions (Union[Unset, UserPermissions]):  Example: {'/': ['*'], '/somedir': ['list', 'download']}.
        used_quota_size (Union[Unset, int]):
        used_quota_files (Union[Unset, int]):
        last_quota_update (Union[Unset, int]): Last quota update as unix timestamp in milliseconds
        upload_bandwidth (Union[Unset, int]): Maximum upload bandwidth as KB/s, 0 means unlimited
        download_bandwidth (Union[Unset, int]): Maximum download bandwidth as KB/s, 0 means unlimited
        upload_data_transfer (Union[Unset, int]): Maximum data transfer allowed for uploads as MB. 0 means no limit
        download_data_transfer (Union[Unset, int]): Maximum data transfer allowed for downloads as MB. 0 means no limit
        total_data_transfer (Union[Unset, int]): Maximum total data transfer as MB. 0 means unlimited. You can set a
            total data transfer instead of the individual values for uploads and downloads
        used_upload_data_transfer (Union[Unset, int]): Uploaded size, as bytes, since the last reset
        used_download_data_transfer (Union[Unset, int]): Downloaded size, as bytes, since the last reset
        created_at (Union[Unset, int]): creation time as unix timestamp in milliseconds. It will be 0 for users created
            before v2.2.0
        updated_at (Union[Unset, int]): last update time as unix timestamp in milliseconds
        last_login (Union[Unset, int]): Last user login as unix timestamp in milliseconds. It is saved at most once
            every 10 minutes
        filters (Union[Unset, UserFilters]): Additional user options
        filesystem (Union[Unset, FilesystemConfig]): Storage filesystem details
        additional_info (Union[Unset, str]): Free form text field for external systems
        oidc_custom_fields (Union[Unset, UserOidcCustomFields]): This field is passed to the pre-login hook if custom
            OIDC token fields have been configured. Field values can be of any type (this is a free form object) and depend
            on the type of the configured OIDC token fields
    """

    id: Union[Unset, int] = UNSET
    status: Union[Unset, UserStatus] = UNSET
    username: Union[Unset, str] = UNSET
    email: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    expiration_date: Union[Unset, int] = UNSET
    password: Union[Unset, str] = UNSET
    public_keys: Union[Unset, List[str]] = UNSET
    home_dir: Union[Unset, str] = UNSET
    virtual_folders: Union[Unset, List[VirtualFolder]] = UNSET
    uid: Union[Unset, int] = UNSET
    gid: Union[Unset, int] = UNSET
    max_sessions: Union[Unset, int] = UNSET
    quota_size: Union[Unset, int] = UNSET
    quota_files: Union[Unset, int] = UNSET
    permissions: Union[Unset, UserPermissions] = UNSET
    used_quota_size: Union[Unset, int] = UNSET
    used_quota_files: Union[Unset, int] = UNSET
    last_quota_update: Union[Unset, int] = UNSET
    upload_bandwidth: Union[Unset, int] = UNSET
    download_bandwidth: Union[Unset, int] = UNSET
    upload_data_transfer: Union[Unset, int] = UNSET
    download_data_transfer: Union[Unset, int] = UNSET
    total_data_transfer: Union[Unset, int] = UNSET
    used_upload_data_transfer: Union[Unset, int] = UNSET
    used_download_data_transfer: Union[Unset, int] = UNSET
    created_at: Union[Unset, int] = UNSET
    updated_at: Union[Unset, int] = UNSET
    last_login: Union[Unset, int] = UNSET
    filters: Union[Unset, UserFilters] = UNSET
    filesystem: Union[Unset, FilesystemConfig] = UNSET
    additional_info: Union[Unset, str] = UNSET
    oidc_custom_fields: Union[Unset, UserOidcCustomFields] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        status: Union[Unset, int] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        username = self.username
        email = self.email
        description = self.description
        expiration_date = self.expiration_date
        password = self.password
        public_keys: Union[Unset, List[str]] = UNSET
        if not isinstance(self.public_keys, Unset):
            public_keys = self.public_keys

        home_dir = self.home_dir
        virtual_folders: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.virtual_folders, Unset):
            virtual_folders = []
            for virtual_folders_item_data in self.virtual_folders:
                virtual_folders_item = virtual_folders_item_data.to_dict()

                virtual_folders.append(virtual_folders_item)

        uid = self.uid
        gid = self.gid
        max_sessions = self.max_sessions
        quota_size = self.quota_size
        quota_files = self.quota_files
        permissions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.permissions, Unset):
            permissions = self.permissions.to_dict()

        used_quota_size = self.used_quota_size
        used_quota_files = self.used_quota_files
        last_quota_update = self.last_quota_update
        upload_bandwidth = self.upload_bandwidth
        download_bandwidth = self.download_bandwidth
        upload_data_transfer = self.upload_data_transfer
        download_data_transfer = self.download_data_transfer
        total_data_transfer = self.total_data_transfer
        used_upload_data_transfer = self.used_upload_data_transfer
        used_download_data_transfer = self.used_download_data_transfer
        created_at = self.created_at
        updated_at = self.updated_at
        last_login = self.last_login
        filters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.filters, Unset):
            filters = self.filters.to_dict()

        filesystem: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.filesystem, Unset):
            filesystem = self.filesystem.to_dict()

        additional_info = self.additional_info
        oidc_custom_fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.oidc_custom_fields, Unset):
            oidc_custom_fields = self.oidc_custom_fields.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if status is not UNSET:
            field_dict["status"] = status
        if username is not UNSET:
            field_dict["username"] = username
        if email is not UNSET:
            field_dict["email"] = email
        if description is not UNSET:
            field_dict["description"] = description
        if expiration_date is not UNSET:
            field_dict["expiration_date"] = expiration_date
        if password is not UNSET:
            field_dict["password"] = password
        if public_keys is not UNSET:
            field_dict["public_keys"] = public_keys
        if home_dir is not UNSET:
            field_dict["home_dir"] = home_dir
        if virtual_folders is not UNSET:
            field_dict["virtual_folders"] = virtual_folders
        if uid is not UNSET:
            field_dict["uid"] = uid
        if gid is not UNSET:
            field_dict["gid"] = gid
        if max_sessions is not UNSET:
            field_dict["max_sessions"] = max_sessions
        if quota_size is not UNSET:
            field_dict["quota_size"] = quota_size
        if quota_files is not UNSET:
            field_dict["quota_files"] = quota_files
        if permissions is not UNSET:
            field_dict["permissions"] = permissions
        if used_quota_size is not UNSET:
            field_dict["used_quota_size"] = used_quota_size
        if used_quota_files is not UNSET:
            field_dict["used_quota_files"] = used_quota_files
        if last_quota_update is not UNSET:
            field_dict["last_quota_update"] = last_quota_update
        if upload_bandwidth is not UNSET:
            field_dict["upload_bandwidth"] = upload_bandwidth
        if download_bandwidth is not UNSET:
            field_dict["download_bandwidth"] = download_bandwidth
        if upload_data_transfer is not UNSET:
            field_dict["upload_data_transfer"] = upload_data_transfer
        if download_data_transfer is not UNSET:
            field_dict["download_data_transfer"] = download_data_transfer
        if total_data_transfer is not UNSET:
            field_dict["total_data_transfer"] = total_data_transfer
        if used_upload_data_transfer is not UNSET:
            field_dict["used_upload_data_transfer"] = used_upload_data_transfer
        if used_download_data_transfer is not UNSET:
            field_dict["used_download_data_transfer"] = used_download_data_transfer
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if last_login is not UNSET:
            field_dict["last_login"] = last_login
        if filters is not UNSET:
            field_dict["filters"] = filters
        if filesystem is not UNSET:
            field_dict["filesystem"] = filesystem
        if additional_info is not UNSET:
            field_dict["additional_info"] = additional_info
        if oidc_custom_fields is not UNSET:
            field_dict["oidc_custom_fields"] = oidc_custom_fields

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, UserStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = UserStatus(_status)

        username = d.pop("username", UNSET)

        email = d.pop("email", UNSET)

        description = d.pop("description", UNSET)

        expiration_date = d.pop("expiration_date", UNSET)

        password = d.pop("password", UNSET)

        public_keys = cast(List[str], d.pop("public_keys", UNSET))

        home_dir = d.pop("home_dir", UNSET)

        virtual_folders = []
        _virtual_folders = d.pop("virtual_folders", UNSET)
        for virtual_folders_item_data in _virtual_folders or []:
            virtual_folders_item = VirtualFolder.from_dict(virtual_folders_item_data)

            virtual_folders.append(virtual_folders_item)

        uid = d.pop("uid", UNSET)

        gid = d.pop("gid", UNSET)

        max_sessions = d.pop("max_sessions", UNSET)

        quota_size = d.pop("quota_size", UNSET)

        quota_files = d.pop("quota_files", UNSET)

        _permissions = d.pop("permissions", UNSET)
        permissions: Union[Unset, UserPermissions]
        if isinstance(_permissions, Unset):
            permissions = UNSET
        else:
            permissions = UserPermissions.from_dict(_permissions)

        used_quota_size = d.pop("used_quota_size", UNSET)

        used_quota_files = d.pop("used_quota_files", UNSET)

        last_quota_update = d.pop("last_quota_update", UNSET)

        upload_bandwidth = d.pop("upload_bandwidth", UNSET)

        download_bandwidth = d.pop("download_bandwidth", UNSET)

        upload_data_transfer = d.pop("upload_data_transfer", UNSET)

        download_data_transfer = d.pop("download_data_transfer", UNSET)

        total_data_transfer = d.pop("total_data_transfer", UNSET)

        used_upload_data_transfer = d.pop("used_upload_data_transfer", UNSET)

        used_download_data_transfer = d.pop("used_download_data_transfer", UNSET)

        created_at = d.pop("created_at", UNSET)

        updated_at = d.pop("updated_at", UNSET)

        last_login = d.pop("last_login", UNSET)

        _filters = d.pop("filters", UNSET)
        filters: Union[Unset, UserFilters]
        if isinstance(_filters, Unset):
            filters = UNSET
        else:
            filters = UserFilters.from_dict(_filters)

        _filesystem = d.pop("filesystem", UNSET)
        filesystem: Union[Unset, FilesystemConfig]
        if isinstance(_filesystem, Unset):
            filesystem = UNSET
        else:
            filesystem = FilesystemConfig.from_dict(_filesystem)

        additional_info = d.pop("additional_info", UNSET)

        _oidc_custom_fields = d.pop("oidc_custom_fields", UNSET)
        oidc_custom_fields: Union[Unset, UserOidcCustomFields]
        if isinstance(_oidc_custom_fields, Unset):
            oidc_custom_fields = UNSET
        else:
            oidc_custom_fields = UserOidcCustomFields.from_dict(_oidc_custom_fields)

        user = cls(
            id=id,
            status=status,
            username=username,
            email=email,
            description=description,
            expiration_date=expiration_date,
            password=password,
            public_keys=public_keys,
            home_dir=home_dir,
            virtual_folders=virtual_folders,
            uid=uid,
            gid=gid,
            max_sessions=max_sessions,
            quota_size=quota_size,
            quota_files=quota_files,
            permissions=permissions,
            used_quota_size=used_quota_size,
            used_quota_files=used_quota_files,
            last_quota_update=last_quota_update,
            upload_bandwidth=upload_bandwidth,
            download_bandwidth=download_bandwidth,
            upload_data_transfer=upload_data_transfer,
            download_data_transfer=download_data_transfer,
            total_data_transfer=total_data_transfer,
            used_upload_data_transfer=used_upload_data_transfer,
            used_download_data_transfer=used_download_data_transfer,
            created_at=created_at,
            updated_at=updated_at,
            last_login=last_login,
            filters=filters,
            filesystem=filesystem,
            additional_info=additional_info,
            oidc_custom_fields=oidc_custom_fields,
        )

        user.additional_properties = d
        return user

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.bandwidth_limit import BandwidthLimit
from ..models.data_transfer_limit import DataTransferLimit
from ..models.hooks_filter import HooksFilter
from ..models.login_methods import LoginMethods
from ..models.mfa_protocols import MFAProtocols
from ..models.patterns_filter import PatternsFilter
from ..models.recovery_code import RecoveryCode
from ..models.supported_protocols import SupportedProtocols
from ..models.user_filters_tls_username import UserFiltersTlsUsername
from ..models.user_totp_config import UserTOTPConfig
from ..models.user_type import UserType
from ..models.web_client_options import WebClientOptions
from ..types import UNSET, Unset

T = TypeVar("T", bound="UserFilters")


@attr.s(auto_attribs=True)
class UserFilters:
    """Additional user options

    Attributes:
        allowed_ip (Union[Unset, List[str]]): only clients connecting from these IP/Mask are allowed. IP/Mask must be in
            CIDR notation as defined in RFC 4632 and RFC 4291, for example "192.0.2.0/24" or "2001:db8::/32" Example:
            ['192.0.2.0/24', '2001:db8::/32'].
        denied_ip (Union[Unset, List[str]]): clients connecting from these IP/Mask are not allowed. Denied rules are
            evaluated before allowed ones Example: ['172.16.0.0/16'].
        denied_login_methods (Union[Unset, List[LoginMethods]]): if null or empty any available login method is allowed
        denied_protocols (Union[Unset, List[SupportedProtocols]]): if null or empty any available protocol is allowed
        file_patterns (Union[Unset, List[PatternsFilter]]): filters based on shell like file patterns. These
            restrictions do not apply to files listing for performance reasons, so a denied file cannot be
            downloaded/overwritten/renamed but it will still be in the list of files. Please note that these restrictions
            can be easily bypassed
        max_upload_file_size (Union[Unset, int]): maximum allowed size, as bytes, for a single file upload. The upload
            will be aborted if/when the size of the file being sent exceeds this limit. 0 means unlimited. This restriction
            does not apply for SSH system commands such as `git` and `rsync`
        tls_username (Union[Unset, UserFiltersTlsUsername]): defines the TLS certificate field to use as username. For
            FTP clients it must match the name provided using the "USER" command. For WebDAV, if no username is provided,
            the CN will be used as username. For WebDAV clients it must match the implicit or provided username. Ignored if
            mutual TLS is disabled
        hooks (Union[Unset, HooksFilter]): User specific hook overrides
        disable_fs_checks (Union[Unset, bool]): Disable checks for existence and automatic creation of home directory
            and virtual folders. SFTPGo requires that the user's home directory, virtual folder root, and intermediate paths
            to virtual folders exist to work properly. If you already know that the required directories exist, disabling
            these checks will speed up login. You could, for example, disable these checks after the first login
        web_client (Union[Unset, List[WebClientOptions]]): WebClient/user REST API related configuration options
        allow_api_key_auth (Union[Unset, bool]): API key authentication allows to impersonate this user with an API key
        user_type (Union[Unset, UserType]): This is an hint for authentication plugins. It is ignored when using SFTPGo
            internal authentication
        totp_config (Union[Unset, UserTOTPConfig]):
        recovery_codes (Union[Unset, List[RecoveryCode]]):
        bandwidth_limits (Union[Unset, List[BandwidthLimit]]):
        data_transfer_limits (Union[Unset, List[DataTransferLimit]]):
        external_auth_cache_time (Union[Unset, int]): Defines the cache time, in seconds, for users authenticated using
            an external auth hook. 0 means no cache
        start_directory (Union[Unset, str]): Specifies an alternate starting directory. If not set, the default is "/".
            This option is supported for SFTP/SCP, FTP and HTTP (WebClient/REST API) protocols. Relative paths will use this
            directory as base.
        field_2fa_protocols (Union[Unset, List[MFAProtocols]]): Defines protocols that require two factor authentication
    """

    allowed_ip: Union[Unset, List[str]] = UNSET
    denied_ip: Union[Unset, List[str]] = UNSET
    denied_login_methods: Union[Unset, List[LoginMethods]] = UNSET
    denied_protocols: Union[Unset, List[SupportedProtocols]] = UNSET
    file_patterns: Union[Unset, List[PatternsFilter]] = UNSET
    max_upload_file_size: Union[Unset, int] = UNSET
    tls_username: Union[Unset, UserFiltersTlsUsername] = UNSET
    hooks: Union[Unset, HooksFilter] = UNSET
    disable_fs_checks: Union[Unset, bool] = UNSET
    web_client: Union[Unset, List[WebClientOptions]] = UNSET
    allow_api_key_auth: Union[Unset, bool] = UNSET
    user_type: Union[Unset, UserType] = UNSET
    totp_config: Union[Unset, UserTOTPConfig] = UNSET
    recovery_codes: Union[Unset, List[RecoveryCode]] = UNSET
    bandwidth_limits: Union[Unset, List[BandwidthLimit]] = UNSET
    data_transfer_limits: Union[Unset, List[DataTransferLimit]] = UNSET
    external_auth_cache_time: Union[Unset, int] = UNSET
    start_directory: Union[Unset, str] = UNSET
    field_2fa_protocols: Union[Unset, List[MFAProtocols]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        allowed_ip: Union[Unset, List[str]] = UNSET
        if not isinstance(self.allowed_ip, Unset):
            allowed_ip = self.allowed_ip

        denied_ip: Union[Unset, List[str]] = UNSET
        if not isinstance(self.denied_ip, Unset):
            denied_ip = self.denied_ip

        denied_login_methods: Union[Unset, List[str]] = UNSET
        if not isinstance(self.denied_login_methods, Unset):
            denied_login_methods = []
            for denied_login_methods_item_data in self.denied_login_methods:
                denied_login_methods_item = denied_login_methods_item_data.value

                denied_login_methods.append(denied_login_methods_item)

        denied_protocols: Union[Unset, List[str]] = UNSET
        if not isinstance(self.denied_protocols, Unset):
            denied_protocols = []
            for denied_protocols_item_data in self.denied_protocols:
                denied_protocols_item = denied_protocols_item_data.value

                denied_protocols.append(denied_protocols_item)

        file_patterns: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.file_patterns, Unset):
            file_patterns = []
            for file_patterns_item_data in self.file_patterns:
                file_patterns_item = file_patterns_item_data.to_dict()

                file_patterns.append(file_patterns_item)

        max_upload_file_size = self.max_upload_file_size
        tls_username: Union[Unset, str] = UNSET
        if not isinstance(self.tls_username, Unset):
            tls_username = self.tls_username.value

        hooks: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.hooks, Unset):
            hooks = self.hooks.to_dict()

        disable_fs_checks = self.disable_fs_checks
        web_client: Union[Unset, List[str]] = UNSET
        if not isinstance(self.web_client, Unset):
            web_client = []
            for web_client_item_data in self.web_client:
                web_client_item = web_client_item_data.value

                web_client.append(web_client_item)

        allow_api_key_auth = self.allow_api_key_auth
        user_type: Union[Unset, str] = UNSET
        if not isinstance(self.user_type, Unset):
            user_type = self.user_type.value

        totp_config: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.totp_config, Unset):
            totp_config = self.totp_config.to_dict()

        recovery_codes: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.recovery_codes, Unset):
            recovery_codes = []
            for recovery_codes_item_data in self.recovery_codes:
                recovery_codes_item = recovery_codes_item_data.to_dict()

                recovery_codes.append(recovery_codes_item)

        bandwidth_limits: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.bandwidth_limits, Unset):
            bandwidth_limits = []
            for bandwidth_limits_item_data in self.bandwidth_limits:
                bandwidth_limits_item = bandwidth_limits_item_data.to_dict()

                bandwidth_limits.append(bandwidth_limits_item)

        data_transfer_limits: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.data_transfer_limits, Unset):
            data_transfer_limits = []
            for data_transfer_limits_item_data in self.data_transfer_limits:
                data_transfer_limits_item = data_transfer_limits_item_data.to_dict()

                data_transfer_limits.append(data_transfer_limits_item)

        external_auth_cache_time = self.external_auth_cache_time
        start_directory = self.start_directory
        field_2fa_protocols: Union[Unset, List[str]] = UNSET
        if not isinstance(self.field_2fa_protocols, Unset):
            field_2fa_protocols = []
            for field_2fa_protocols_item_data in self.field_2fa_protocols:
                field_2fa_protocols_item = field_2fa_protocols_item_data.value

                field_2fa_protocols.append(field_2fa_protocols_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if allowed_ip is not UNSET:
            field_dict["allowed_ip"] = allowed_ip
        if denied_ip is not UNSET:
            field_dict["denied_ip"] = denied_ip
        if denied_login_methods is not UNSET:
            field_dict["denied_login_methods"] = denied_login_methods
        if denied_protocols is not UNSET:
            field_dict["denied_protocols"] = denied_protocols
        if file_patterns is not UNSET:
            field_dict["file_patterns"] = file_patterns
        if max_upload_file_size is not UNSET:
            field_dict["max_upload_file_size"] = max_upload_file_size
        if tls_username is not UNSET:
            field_dict["tls_username"] = tls_username
        if hooks is not UNSET:
            field_dict["hooks"] = hooks
        if disable_fs_checks is not UNSET:
            field_dict["disable_fs_checks"] = disable_fs_checks
        if web_client is not UNSET:
            field_dict["web_client"] = web_client
        if allow_api_key_auth is not UNSET:
            field_dict["allow_api_key_auth"] = allow_api_key_auth
        if user_type is not UNSET:
            field_dict["user_type"] = user_type
        if totp_config is not UNSET:
            field_dict["totp_config"] = totp_config
        if recovery_codes is not UNSET:
            field_dict["recovery_codes"] = recovery_codes
        if bandwidth_limits is not UNSET:
            field_dict["bandwidth_limits"] = bandwidth_limits
        if data_transfer_limits is not UNSET:
            field_dict["data_transfer_limits"] = data_transfer_limits
        if external_auth_cache_time is not UNSET:
            field_dict["external_auth_cache_time"] = external_auth_cache_time
        if start_directory is not UNSET:
            field_dict["start_directory"] = start_directory
        if field_2fa_protocols is not UNSET:
            field_dict["2fa_protocols"] = field_2fa_protocols

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        allowed_ip = cast(List[str], d.pop("allowed_ip", UNSET))

        denied_ip = cast(List[str], d.pop("denied_ip", UNSET))

        denied_login_methods = []
        _denied_login_methods = d.pop("denied_login_methods", UNSET)
        for denied_login_methods_item_data in _denied_login_methods or []:
            denied_login_methods_item = LoginMethods(denied_login_methods_item_data)

            denied_login_methods.append(denied_login_methods_item)

        denied_protocols = []
        _denied_protocols = d.pop("denied_protocols", UNSET)
        for denied_protocols_item_data in _denied_protocols or []:
            denied_protocols_item = SupportedProtocols(denied_protocols_item_data)

            denied_protocols.append(denied_protocols_item)

        file_patterns = []
        _file_patterns = d.pop("file_patterns", UNSET)
        for file_patterns_item_data in _file_patterns or []:
            file_patterns_item = PatternsFilter.from_dict(file_patterns_item_data)

            file_patterns.append(file_patterns_item)

        max_upload_file_size = d.pop("max_upload_file_size", UNSET)

        _tls_username = d.pop("tls_username", UNSET)
        tls_username: Union[Unset, UserFiltersTlsUsername]
        if isinstance(_tls_username, Unset):
            tls_username = UNSET
        else:
            tls_username = UserFiltersTlsUsername(_tls_username)

        _hooks = d.pop("hooks", UNSET)
        hooks: Union[Unset, HooksFilter]
        if isinstance(_hooks, Unset):
            hooks = UNSET
        else:
            hooks = HooksFilter.from_dict(_hooks)

        disable_fs_checks = d.pop("disable_fs_checks", UNSET)

        web_client = []
        _web_client = d.pop("web_client", UNSET)
        for web_client_item_data in _web_client or []:
            web_client_item = WebClientOptions(web_client_item_data)

            web_client.append(web_client_item)

        allow_api_key_auth = d.pop("allow_api_key_auth", UNSET)

        _user_type = d.pop("user_type", UNSET)
        user_type: Union[Unset, UserType]
        if isinstance(_user_type, Unset):
            user_type = UNSET
        else:
            user_type = UserType(_user_type)

        _totp_config = d.pop("totp_config", UNSET)
        totp_config: Union[Unset, UserTOTPConfig]
        if isinstance(_totp_config, Unset):
            totp_config = UNSET
        else:
            totp_config = UserTOTPConfig.from_dict(_totp_config)

        recovery_codes = []
        _recovery_codes = d.pop("recovery_codes", UNSET)
        for recovery_codes_item_data in _recovery_codes or []:
            recovery_codes_item = RecoveryCode.from_dict(recovery_codes_item_data)

            recovery_codes.append(recovery_codes_item)

        bandwidth_limits = []
        _bandwidth_limits = d.pop("bandwidth_limits", UNSET)
        for bandwidth_limits_item_data in _bandwidth_limits or []:
            bandwidth_limits_item = BandwidthLimit.from_dict(bandwidth_limits_item_data)

            bandwidth_limits.append(bandwidth_limits_item)

        data_transfer_limits = []
        _data_transfer_limits = d.pop("data_transfer_limits", UNSET)
        for data_transfer_limits_item_data in _data_transfer_limits or []:
            data_transfer_limits_item = DataTransferLimit.from_dict(
                data_transfer_limits_item_data
            )

            data_transfer_limits.append(data_transfer_limits_item)

        external_auth_cache_time = d.pop("external_auth_cache_time", UNSET)

        start_directory = d.pop("start_directory", UNSET)

        field_2fa_protocols = []
        _field_2fa_protocols = d.pop("2fa_protocols", UNSET)
        for field_2fa_protocols_item_data in _field_2fa_protocols or []:
            field_2fa_protocols_item = MFAProtocols(field_2fa_protocols_item_data)

            field_2fa_protocols.append(field_2fa_protocols_item)

        user_filters = cls(
            allowed_ip=allowed_ip,
            denied_ip=denied_ip,
            denied_login_methods=denied_login_methods,
            denied_protocols=denied_protocols,
            file_patterns=file_patterns,
            max_upload_file_size=max_upload_file_size,
            tls_username=tls_username,
            hooks=hooks,
            disable_fs_checks=disable_fs_checks,
            web_client=web_client,
            allow_api_key_auth=allow_api_key_auth,
            user_type=user_type,
            totp_config=totp_config,
            recovery_codes=recovery_codes,
            bandwidth_limits=bandwidth_limits,
            data_transfer_limits=data_transfer_limits,
            external_auth_cache_time=external_auth_cache_time,
            start_directory=start_directory,
            field_2fa_protocols=field_2fa_protocols,
        )

        user_filters.additional_properties = d
        return user_filters

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .add_api_key_response_201 import AddApiKeyResponse201
from .admin import Admin
from .admin_filters import AdminFilters
from .admin_permissions import AdminPermissions
from .admin_profile import AdminProfile
from .admin_reset_password_json_body import AdminResetPasswordJsonBody
from .admin_status import AdminStatus
from .api_key import APIKey
from .api_key_scope import APIKeyScope
from .api_response import ApiResponse
from .azure_blob_fs_config import AzureBlobFsConfig
from .azure_blob_fs_config_access_tier import AzureBlobFsConfigAccessTier
from .backup_data import BackupData
from .ban_status import BanStatus
from .bandwidth_limit import BandwidthLimit
from .base_totp_config import BaseTOTPConfig
from .base_virtual_folder import BaseVirtualFolder
from .connection_status import ConnectionStatus
from .connection_status_protocol import ConnectionStatusProtocol
from .create_user_files_multipart_data import CreateUserFilesMultipartData
from .crypt_fs_config import CryptFsConfig
from .data_provider_status import DataProviderStatus
from .data_transfer_limit import DataTransferLimit
from .defender_entry import DefenderEntry
from .dir_entry import DirEntry
from .dir_permissions import DirPermissions
from .dumpdata_indent import DumpdataIndent
from .dumpdata_output_data import DumpdataOutputData
from .event_protocols import EventProtocols
from .filesystem_config import FilesystemConfig
from .folder_quota_scan import FolderQuotaScan
from .folder_quota_update_usage_deprecated_mode import (
    FolderQuotaUpdateUsageDeprecatedMode,
)
from .folder_quota_update_usage_mode import FolderQuotaUpdateUsageMode
from .folder_retention import FolderRetention
from .fs_event import FsEvent
from .fs_event_action import FsEventAction
from .fs_event_status import FsEventStatus
from .fs_providers import FsProviders
from .ftp_passive_port_range import FTPPassivePortRange
from .ftp_service_status import FTPServiceStatus
from .ftpd_binding import FTPDBinding
from .ftpd_binding_active_connections_security import (
    FTPDBindingActiveConnectionsSecurity,
)
from .ftpd_binding_passive_connections_security import (
    FTPDBindingPassiveConnectionsSecurity,
)
from .ftpd_binding_tls_mode import FTPDBindingTlsMode
from .gcs_config import GCSConfig
from .gcs_config_automatic_credentials import GCSConfigAutomaticCredentials
from .generate_admin_totp_secret_json_body import GenerateAdminTotpSecretJsonBody
from .generate_admin_totp_secret_response_200 import GenerateAdminTotpSecretResponse200
from .generate_user_totp_secret_json_body import GenerateUserTotpSecretJsonBody
from .generate_user_totp_secret_response_200 import GenerateUserTotpSecretResponse200
from .get_admins_order import GetAdminsOrder
from .get_api_keys_order import GetApiKeysOrder
from .get_folders_order import GetFoldersOrder
from .get_fs_events_order import GetFsEventsOrder
from .get_provider_events_order import GetProviderEventsOrder
from .get_user_shares_order import GetUserSharesOrder
from .get_users_order import GetUsersOrder
from .hooks_filter import HooksFilter
from .loaddata_from_file_mode import LoaddataFromFileMode
from .loaddata_from_file_scan_quota import LoaddataFromFileScanQuota
from .loaddata_from_request_body_mode import LoaddataFromRequestBodyMode
from .loaddata_from_request_body_scan_quota import LoaddataFromRequestBodyScanQuota
from .login_methods import LoginMethods
from .metadata_check import MetadataCheck
from .mfa_protocols import MFAProtocols
from .mfa_status import MFAStatus
from .passive_ip_override import PassiveIPOverride
from .patterns_filter import PatternsFilter
from .patterns_filter_deny_policy import PatternsFilterDenyPolicy
from .permission import Permission
from .provider_event import ProviderEvent
from .provider_event_action import ProviderEventAction
from .provider_event_object_type import ProviderEventObjectType
from .pwd_change import PwdChange
from .quota_scan import QuotaScan
from .quota_usage import QuotaUsage
from .recovery_code import RecoveryCode
from .retention_check import RetentionCheck
from .retention_check_notification import RetentionCheckNotification
from .s3_config import S3Config
from .score_status import ScoreStatus
from .secret import Secret
from .secret_status import SecretStatus
from .services_status import ServicesStatus
from .services_status_defender import ServicesStatusDefender
from .setprops_user_file_json_body import SetpropsUserFileJsonBody
from .sftp_fs_config import SFTPFsConfig
from .share import Share
from .share_scope import ShareScope
from .ssh_authentications import SSHAuthentications
from .ssh_binding import SSHBinding
from .ssh_host_key import SSHHostKey
from .ssh_service_status import SSHServiceStatus
from .supported_protocols import SupportedProtocols
from .tls_versions import TLSVersions
from .token import Token
from .totp_config import TOTPConfig
from .totph_mac_algo import TOTPHMacAlgo
from .transfer import Transfer
from .transfer_operation_type import TransferOperationType
from .transfer_quota_usage import TransferQuotaUsage
from .unban_host_json_body import UnbanHostJsonBody
from .update_user_disconnect import UpdateUserDisconnect
from .upload_to_share_multipart_data import UploadToShareMultipartData
from .user import User
from .user_filters import UserFilters
from .user_filters_tls_username import UserFiltersTlsUsername
from .user_oidc_custom_fields import UserOidcCustomFields
from .user_permissions import UserPermissions
from .user_profile import UserProfile
from .user_quota_update_usage_deprecated_mode import UserQuotaUpdateUsageDeprecatedMode
from .user_quota_update_usage_mode import UserQuotaUpdateUsageMode
from .user_reset_password_json_body import UserResetPasswordJsonBody
from .user_status import UserStatus
from .user_totp_config import UserTOTPConfig
from .user_transfer_quota_update_usage_mode import UserTransferQuotaUpdateUsageMode
from .user_type import UserType
from .validate_admin_totp_secret_json_body import ValidateAdminTotpSecretJsonBody
from .validate_user_totp_secret_json_body import ValidateUserTotpSecretJsonBody
from .version_info import VersionInfo
from .virtual_folder import VirtualFolder
from .web_client_options import WebClientOptions
from .web_dav_binding import WebDAVBinding
from .web_dav_service_status import WebDAVServiceStatus



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="AdminProfile")


@attr.s(auto_attribs=True)
class AdminProfile:
    """
    Attributes:
        email (Union[Unset, str]):
        description (Union[Unset, str]):
        allow_api_key_auth (Union[Unset, bool]): If enabled, you can impersonate this admin, in REST API, using an API
            key. If disabled admin credentials are required for impersonation
    """

    email: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    allow_api_key_auth: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        email = self.email
        description = self.description
        allow_api_key_auth = self.allow_api_key_auth

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if email is not UNSET:
            field_dict["email"] = email
        if description is not UNSET:
            field_dict["description"] = description
        if allow_api_key_auth is not UNSET:
            field_dict["allow_api_key_auth"] = allow_api_key_auth

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        email = d.pop("email", UNSET)

        description = d.pop("description", UNSET)

        allow_api_key_auth = d.pop("allow_api_key_auth", UNSET)

        admin_profile = cls(
            email=email,
            description=description,
            allow_api_key_auth=allow_api_key_auth,
        )

        admin_profile.additional_properties = d
        return admin_profile

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import json
from io import BytesIO
from typing import Any, Dict, List, Tuple, Type, TypeVar, Union

import attr

from ..types import UNSET, File, FileJsonType, Unset

T = TypeVar("T", bound="UploadToShareMultipartData")


@attr.s(auto_attribs=True)
class UploadToShareMultipartData:
    """
    Attributes:
        filenames (Union[Unset, List[File]]):
    """

    filenames: Union[Unset, List[File]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        filenames: Union[Unset, List[FileJsonType]] = UNSET
        if not isinstance(self.filenames, Unset):
            filenames = []
            for filenames_item_data in self.filenames:
                filenames_item = filenames_item_data.to_tuple()

                filenames.append(filenames_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if filenames is not UNSET:
            field_dict["filenames"] = filenames

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        filenames: Union[Unset, Tuple[None, bytes, str]] = UNSET
        if not isinstance(self.filenames, Unset):
            _temp_filenames = []
            for filenames_item_data in self.filenames:
                filenames_item = filenames_item_data.to_tuple()

                _temp_filenames.append(filenames_item)
            filenames = (None, json.dumps(_temp_filenames).encode(), "application/json")

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                key: (None, str(value).encode(), "text/plain")
                for key, value in self.additional_properties.items()
            }
        )
        field_dict.update({})
        if filenames is not UNSET:
            field_dict["filenames"] = filenames

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        filenames = []
        _filenames = d.pop("filenames", UNSET)
        for filenames_item_data in _filenames or []:
            filenames_item = File(payload=BytesIO(filenames_item_data))

            filenames.append(filenames_item)

        upload_to_share_multipart_data = cls(
            filenames=filenames,
        )

        upload_to_share_multipart_data.additional_properties = d
        return upload_to_share_multipart_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.mfa_protocols import MFAProtocols
from ..models.secret import Secret
from ..types import UNSET, Unset

T = TypeVar("T", bound="UserTOTPConfig")


@attr.s(auto_attribs=True)
class UserTOTPConfig:
    """
    Attributes:
        enabled (Union[Unset, bool]):
        config_name (Union[Unset, str]): This name must be defined within the "totp" section of the SFTPGo configuration
            file. You will be unable to save a user/admin referencing a missing config_name
        secret (Union[Unset, Secret]): The secret is encrypted before saving, so to set a new secret you must provide a
            payload and set the status to "Plain". The encryption key and additional data will be generated automatically.
            If you set the status to "Redacted" the existig secret will be preserved
        protocols (Union[Unset, List[MFAProtocols]]): TOTP will be required for the specified protocols. SSH protocol
            (SFTP/SCP/SSH commands) will ask for the TOTP passcode if the client uses keyboard interactive authentication.
            FTP has no standard way to support two factor authentication, if you enable the FTP support, you have to add the
            TOTP passcode after the password. For example if your password is "password" and your one time passcode is
            "123456" you have to use "password123456" as password. WebDAV is not supported since each single request must be
            authenticated and a passcode cannot be reused.
    """

    enabled: Union[Unset, bool] = UNSET
    config_name: Union[Unset, str] = UNSET
    secret: Union[Unset, Secret] = UNSET
    protocols: Union[Unset, List[MFAProtocols]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        enabled = self.enabled
        config_name = self.config_name
        secret: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.secret, Unset):
            secret = self.secret.to_dict()

        protocols: Union[Unset, List[str]] = UNSET
        if not isinstance(self.protocols, Unset):
            protocols = []
            for protocols_item_data in self.protocols:
                protocols_item = protocols_item_data.value

                protocols.append(protocols_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if config_name is not UNSET:
            field_dict["config_name"] = config_name
        if secret is not UNSET:
            field_dict["secret"] = secret
        if protocols is not UNSET:
            field_dict["protocols"] = protocols

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        enabled = d.pop("enabled", UNSET)

        config_name = d.pop("config_name", UNSET)

        _secret = d.pop("secret", UNSET)
        secret: Union[Unset, Secret]
        if isinstance(_secret, Unset):
            secret = UNSET
        else:
            secret = Secret.from_dict(_secret)

        protocols = []
        _protocols = d.pop("protocols", UNSET)
        for protocols_item_data in _protocols or []:
            protocols_item = MFAProtocols(protocols_item_data)

            protocols.append(protocols_item)

        user_totp_config = cls(
            enabled=enabled,
            config_name=config_name,
            secret=secret,
            protocols=protocols,
        )

        user_totp_config.additional_properties = d
        return user_totp_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GenerateUserTotpSecretResponse200")


@attr.s(auto_attribs=True)
class GenerateUserTotpSecretResponse200:
    """
    Attributes:
        config_name (Union[Unset, str]):
        issuer (Union[Unset, str]):
        secret (Union[Unset, str]):
        qr_code (Union[Unset, str]): QR code png encoded as BASE64
    """

    config_name: Union[Unset, str] = UNSET
    issuer: Union[Unset, str] = UNSET
    secret: Union[Unset, str] = UNSET
    qr_code: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        config_name = self.config_name
        issuer = self.issuer
        secret = self.secret
        qr_code = self.qr_code

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if config_name is not UNSET:
            field_dict["config_name"] = config_name
        if issuer is not UNSET:
            field_dict["issuer"] = issuer
        if secret is not UNSET:
            field_dict["secret"] = secret
        if qr_code is not UNSET:
            field_dict["qr_code"] = qr_code

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        config_name = d.pop("config_name", UNSET)

        issuer = d.pop("issuer", UNSET)

        secret = d.pop("secret", UNSET)

        qr_code = d.pop("qr_code", UNSET)

        generate_user_totp_secret_response_200 = cls(
            config_name=config_name,
            issuer=issuer,
            secret=secret,
            qr_code=qr_code,
        )

        generate_user_totp_secret_response_200.additional_properties = d
        return generate_user_totp_secret_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="DefenderEntry")


@attr.s(auto_attribs=True)
class DefenderEntry:
    """
    Attributes:
        id (Union[Unset, str]):
        ip (Union[Unset, str]):
        score (Union[Unset, int]): the score increases whenever a violation is detected, such as an attempt to log in
            using an incorrect password or invalid username. If the score exceeds the configured threshold, the IP is
            banned. Omitted for banned IPs
        ban_time (Union[Unset, datetime.datetime]): date time until the IP is banned. For already banned hosts, the ban
            time is increased each time a new violation is detected. Omitted if the IP is not banned
    """

    id: Union[Unset, str] = UNSET
    ip: Union[Unset, str] = UNSET
    score: Union[Unset, int] = UNSET
    ban_time: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        ip = self.ip
        score = self.score
        ban_time: Union[Unset, str] = UNSET
        if not isinstance(self.ban_time, Unset):
            ban_time = self.ban_time.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if ip is not UNSET:
            field_dict["ip"] = ip
        if score is not UNSET:
            field_dict["score"] = score
        if ban_time is not UNSET:
            field_dict["ban_time"] = ban_time

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        ip = d.pop("ip", UNSET)

        score = d.pop("score", UNSET)

        _ban_time = d.pop("ban_time", UNSET)
        ban_time: Union[Unset, datetime.datetime]
        if isinstance(_ban_time, Unset):
            ban_time = UNSET
        else:
            ban_time = isoparse(_ban_time)

        defender_entry = cls(
            id=id,
            ip=ip,
            score=score,
            ban_time=ban_time,
        )

        defender_entry.additional_properties = d
        return defender_entry

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="DataTransferLimit")


@attr.s(auto_attribs=True)
class DataTransferLimit:
    """
    Attributes:
        sources (Union[Unset, List[str]]): Source networks in CIDR notation as defined in RFC 4632 and RFC 4291 for
            example `192.0.2.0/24` or `2001:db8::/32`. The limit applies if the defined networks contain the client IP
        upload_data_transfer (Union[Unset, int]): Maximum data transfer allowed for uploads as MB. 0 means no limit
        download_data_transfer (Union[Unset, int]): Maximum data transfer allowed for downloads as MB. 0 means no limit
        total_data_transfer (Union[Unset, int]): Maximum total data transfer as MB. 0 means unlimited. You can set a
            total data transfer instead of the individual values for uploads and downloads
    """

    sources: Union[Unset, List[str]] = UNSET
    upload_data_transfer: Union[Unset, int] = UNSET
    download_data_transfer: Union[Unset, int] = UNSET
    total_data_transfer: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        sources: Union[Unset, List[str]] = UNSET
        if not isinstance(self.sources, Unset):
            sources = self.sources

        upload_data_transfer = self.upload_data_transfer
        download_data_transfer = self.download_data_transfer
        total_data_transfer = self.total_data_transfer

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if sources is not UNSET:
            field_dict["sources"] = sources
        if upload_data_transfer is not UNSET:
            field_dict["upload_data_transfer"] = upload_data_transfer
        if download_data_transfer is not UNSET:
            field_dict["download_data_transfer"] = download_data_transfer
        if total_data_transfer is not UNSET:
            field_dict["total_data_transfer"] = total_data_transfer

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        sources = cast(List[str], d.pop("sources", UNSET))

        upload_data_transfer = d.pop("upload_data_transfer", UNSET)

        download_data_transfer = d.pop("download_data_transfer", UNSET)

        total_data_transfer = d.pop("total_data_transfer", UNSET)

        data_transfer_limit = cls(
            sources=sources,
            upload_data_transfer=upload_data_transfer,
            download_data_transfer=download_data_transfer,
            total_data_transfer=total_data_transfer,
        )

        data_transfer_limit.additional_properties = d
        return data_transfer_limit

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List

import httpx

from ...client import Client
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
    json_body: List[str],
) -> Dict[str, Any]:
    url = "{}/user/streamzip".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_json_body = json_body

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _build_response(*, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=None,
    )


def sync_detailed(
    *,
    client: Client,
    json_body: List[str],
) -> Response[Any]:
    """Download multiple files and folders as a single zip file

     A zip file, containing the specified files and folders, will be generated on the fly and returned as
    response body. Only folders and regular files will be included in the zip

    Args:
        json_body (List[str]):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


async def asyncio_detailed(
    *,
    client: Client,
    json_body: List[str],
) -> Response[Any]:
    """Download multiple files and folders as a single zip file

     A zip file, containing the specified files and folders, will be generated on the fly and returned as
    response body. Only folders and regular files will be included in the zip

    Args:
        json_body (List[str]):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)



from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ...client import AuthenticatedClient
from ...models.api_response import ApiResponse
from ...models.upload_to_share_multipart_data import UploadToShareMultipartData
from ...types import Response


def _get_kwargs(
    id: str,
    *,
    client: AuthenticatedClient,
    multipart_data: UploadToShareMultipartData,
) -> Dict[str, Any]:
    url = "{}/shares/{id}".format(client.base_url, id=id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    multipart_multipart_data = multipart_data.to_multipart()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "files": multipart_multipart_data,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[Any, List[ApiResponse]]]:
    if response.status_code == 201:
        response_201 = []
        _response_201 = response.json()
        for response_201_item_data in _response_201:
            response_201_item = ApiResponse.from_dict(response_201_item_data)

            response_201.append(response_201_item)

        return response_201
    if response.status_code == 400:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == 401:
        response_401 = cast(Any, None)
        return response_401
    if response.status_code == 403:
        response_403 = cast(Any, None)
        return response_403
    if response.status_code == 413:
        response_413 = cast(Any, None)
        return response_413
    if response.status_code == 500:
        response_500 = cast(Any, None)
        return response_500
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[Union[Any, List[ApiResponse]]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    multipart_data: UploadToShareMultipartData,
) -> Response[Union[Any, List[ApiResponse]]]:
    """Upload one or more files to the shared path

     The share must be defined with the write scope and the associated user must have the upload
    permission

    Args:
        id (str):
        multipart_data (UploadToShareMultipartData):

    Returns:
        Response[Union[Any, List[ApiResponse]]]
    """

    kwargs = _get_kwargs(
        id=id,
        client=client,
        multipart_data=multipart_data,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    id: str,
    *,
    client: AuthenticatedClient,
    multipart_data: UploadToShareMultipartData,
) -> Optional[Union[Any, List[ApiResponse]]]:
    """Upload one or more files to the shared path

     The share must be defined with the write scope and the associated user must have the upload
    permission

    Args:
        id (str):
        multipart_data (UploadToShareMultipartData):

    Returns:
        Response[Union[Any, List[ApiResponse]]]
    """

    return sync_detailed(
        id=id,
        client=client,
        multipart_data=multipart_data,
    ).parsed


async def asyncio_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    multipart_data: UploadToShareMultipartData,
) -> Response[Union[Any, List[ApiResponse]]]:
    """Upload one or more files to the shared path

     The share must be defined with the write scope and the associated user must have the upload
    permission

    Args:
        id (str):
        multipart_data (UploadToShareMultipartData):

    Returns:
        Response[Union[Any, List[ApiResponse]]]
    """

    kwargs = _get_kwargs(
        id=id,
        client=client,
        multipart_data=multipart_data,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    id: str,
    *,
    client: AuthenticatedClient,
    multipart_data: UploadToShareMultipartData,
) -> Optional[Union[Any, List[ApiResponse]]]:
    """Upload one or more files to the shared path

     The share must be defined with the write scope and the associated user must have the upload
    permission

    Args:
        id (str):
        multipart_data (UploadToShareMultipartData):

    Returns:
        Response[Union[Any, List[ApiResponse]]]
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
            multipart_data=multipart_data,
        )
    ).parsed



from typing import Any, Dict, Optional, Union, cast

import httpx

from ...client import Client
from ...models.api_response import ApiResponse
from ...types import Response


def _get_kwargs(
    username: str,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/admins/{username}/2fa/disable".format(client.base_url, username=username)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "put",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[Any, ApiResponse]]:
    if response.status_code == 200:
        response_200 = ApiResponse.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == 401:
        response_401 = cast(Any, None)
        return response_401
    if response.status_code == 403:
        response_403 = cast(Any, None)
        return response_403
    if response.status_code == 404:
        response_404 = cast(Any, None)
        return response_404
    if response.status_code == 500:
        response_500 = cast(Any, None)
        return response_500
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[Any, ApiResponse]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    username: str,
    *,
    client: Client,
) -> Response[Union[Any, ApiResponse]]:
    """Disable second factor authentication

     Disables second factor authentication for the given admin. This API must be used if the admin loses
    access to their second factor auth device and has no recovery codes

    Args:
        username (str):

    Returns:
        Response[Union[Any, ApiResponse]]
    """

    kwargs = _get_kwargs(
        username=username,
        client=client,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    username: str,
    *,
    client: Client,
) -> Optional[Union[Any, ApiResponse]]:
    """Disable second factor authentication

     Disables second factor authentication for the given admin. This API must be used if the admin loses
    access to their second factor auth device and has no recovery codes

    Args:
        username (str):

    Returns:
        Response[Union[Any, ApiResponse]]
    """

    return sync_detailed(
        username=username,
        client=client,
    ).parsed


async def asyncio_detailed(
    username: str,
    *,
    client: Client,
) -> Response[Union[Any, ApiResponse]]:
    """Disable second factor authentication

     Disables second factor authentication for the given admin. This API must be used if the admin loses
    access to their second factor auth device and has no recovery codes

    Args:
        username (str):

    Returns:
        Response[Union[Any, ApiResponse]]
    """

    kwargs = _get_kwargs(
        username=username,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    username: str,
    *,
    client: Client,
) -> Optional[Union[Any, ApiResponse]]:
    """Disable second factor authentication

     Disables second factor authentication for the given admin. This API must be used if the admin loses
    access to their second factor auth device and has no recovery codes

    Args:
        username (str):

    Returns:
        Response[Union[Any, ApiResponse]]
    """

    return (
        await asyncio_detailed(
            username=username,
            client=client,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.api_key_scope import APIKeyScope
from ..types import UNSET, Unset

T = TypeVar("T", bound="APIKey")


@attr.s(auto_attribs=True)
class APIKey:
    """
    Attributes:
        id (Union[Unset, str]): unique key identifier
        name (Union[Unset, str]): User friendly key name
        key (Union[Unset, str]): We store the hash of the key. This is just like a password. For security reasons this
            field is omitted when you search/get API keys
        scope (Union[Unset, APIKeyScope]): Options:
              * `1` - admin scope. The API key will be used to impersonate an SFTPGo admin
              * `2` - user scope. The API key will be used to impersonate an SFTPGo user
        created_at (Union[Unset, int]): creation time as unix timestamp in milliseconds
        updated_at (Union[Unset, int]): last update time as unix timestamp in milliseconds
        last_use_at (Union[Unset, int]): last use time as unix timestamp in milliseconds. It is saved at most once every
            10 minutes
        expires_at (Union[Unset, int]): expiration time as unix timestamp in milliseconds
        description (Union[Unset, str]): optional description
        user (Union[Unset, str]): username associated with this API key. If empty and the scope is "user scope" the key
            can impersonate any user
        admin (Union[Unset, str]): admin associated with this API key. If empty and the scope is "admin scope" the key
            can impersonate any admin
    """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    key: Union[Unset, str] = UNSET
    scope: Union[Unset, APIKeyScope] = UNSET
    created_at: Union[Unset, int] = UNSET
    updated_at: Union[Unset, int] = UNSET
    last_use_at: Union[Unset, int] = UNSET
    expires_at: Union[Unset, int] = UNSET
    description: Union[Unset, str] = UNSET
    user: Union[Unset, str] = UNSET
    admin: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        name = self.name
        key = self.key
        scope: Union[Unset, int] = UNSET
        if not isinstance(self.scope, Unset):
            scope = self.scope.value

        created_at = self.created_at
        updated_at = self.updated_at
        last_use_at = self.last_use_at
        expires_at = self.expires_at
        description = self.description
        user = self.user
        admin = self.admin

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if key is not UNSET:
            field_dict["key"] = key
        if scope is not UNSET:
            field_dict["scope"] = scope
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if last_use_at is not UNSET:
            field_dict["last_use_at"] = last_use_at
        if expires_at is not UNSET:
            field_dict["expires_at"] = expires_at
        if description is not UNSET:
            field_dict["description"] = description
        if user is not UNSET:
            field_dict["user"] = user
        if admin is not UNSET:
            field_dict["admin"] = admin

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        key = d.pop("key", UNSET)

        _scope = d.pop("scope", UNSET)
        scope: Union[Unset, APIKeyScope]
        if isinstance(_scope, Unset):
            scope = UNSET
        else:
            scope = APIKeyScope(_scope)

        created_at = d.pop("created_at", UNSET)

        updated_at = d.pop("updated_at", UNSET)

        last_use_at = d.pop("last_use_at", UNSET)

        expires_at = d.pop("expires_at", UNSET)

        description = d.pop("description", UNSET)

        user = d.pop("user", UNSET)

        admin = d.pop("admin", UNSET)

        api_key = cls(
            id=id,
            name=name,
            key=key,
            scope=scope,
            created_at=created_at,
            updated_at=updated_at,
            last_use_at=last_use_at,
            expires_at=expires_at,
            description=description,
            user=user,
            admin=admin,
        )

        api_key.additional_properties = d
        return api_key

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="FolderRetention")


@attr.s(auto_attribs=True)
class FolderRetention:
    """
    Attributes:
        path (Union[Unset, str]): exposed virtual directory path, if no other specific retention is defined, the
            retention applies for sub directories too. For example if retention is defined for the paths "/" and "/sub" then
            the retention for "/" is applied for any file outside the "/sub" directory Example: /.
        retention (Union[Unset, int]): retention time in hours. All the files with a modification time older than the
            defined value will be deleted. 0 means exclude this path Example: 24.
        delete_empty_dirs (Union[Unset, bool]): if enabled, empty directories will be deleted
        ignore_user_permissions (Union[Unset, bool]): if enabled, files will be deleted even if the user does not have
            the delete permission. The default is "false" which means that files will be skipped if the user does not have
            permission to delete them. File patterns filters will always be silently ignored
    """

    path: Union[Unset, str] = UNSET
    retention: Union[Unset, int] = UNSET
    delete_empty_dirs: Union[Unset, bool] = UNSET
    ignore_user_permissions: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        path = self.path
        retention = self.retention
        delete_empty_dirs = self.delete_empty_dirs
        ignore_user_permissions = self.ignore_user_permissions

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if path is not UNSET:
            field_dict["path"] = path
        if retention is not UNSET:
            field_dict["retention"] = retention
        if delete_empty_dirs is not UNSET:
            field_dict["delete_empty_dirs"] = delete_empty_dirs
        if ignore_user_permissions is not UNSET:
            field_dict["ignore_user_permissions"] = ignore_user_permissions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        path = d.pop("path", UNSET)

        retention = d.pop("retention", UNSET)

        delete_empty_dirs = d.pop("delete_empty_dirs", UNSET)

        ignore_user_permissions = d.pop("ignore_user_permissions", UNSET)

        folder_retention = cls(
            path=path,
            retention=retention,
            delete_empty_dirs=delete_empty_dirs,
            ignore_user_permissions=ignore_user_permissions,
        )

        folder_retention.additional_properties = d
        return folder_retention

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.secret import Secret
from ..types import UNSET, Unset

T = TypeVar("T", bound="CryptFsConfig")


@attr.s(auto_attribs=True)
class CryptFsConfig:
    """Crypt filesystem configuration details

    Attributes:
        passphrase (Union[Unset, Secret]): The secret is encrypted before saving, so to set a new secret you must
            provide a payload and set the status to "Plain". The encryption key and additional data will be generated
            automatically. If you set the status to "Redacted" the existig secret will be preserved
    """

    passphrase: Union[Unset, Secret] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        passphrase: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.passphrase, Unset):
            passphrase = self.passphrase.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if passphrase is not UNSET:
            field_dict["passphrase"] = passphrase

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _passphrase = d.pop("passphrase", UNSET)
        passphrase: Union[Unset, Secret]
        if isinstance(_passphrase, Unset):
            passphrase = UNSET
        else:
            passphrase = Secret.from_dict(_passphrase)

        crypt_fs_config = cls(
            passphrase=passphrase,
        )

        crypt_fs_config.additional_properties = d
        return crypt_fs_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class FsEventAction(str, Enum):
    DOWNLOAD = "download"
    PRE_UPLOAD = "pre-upload"
    UPLOAD = "upload"
    DELETE = "delete"
    RENAME = "rename"
    MKDIR = "mkdir"
    RMDIR = "rmdir"
    SSH_CMD = "ssh_cmd"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class EventProtocols(str, Enum):
    SSH = "SSH"
    SFTP = "SFTP"
    SCP = "SCP"
    FTP = "FTP"
    DAV = "DAV"
    HTTP = "HTTP"
    HTTPSHARE = "HTTPShare"
    DATARETENTION = "DataRetention"
    OIDC = "OIDC"

    def __str__(self) -> str:
        return str(self.value)
